
.content .fa, .content .bi
  color: #555

.top-part
  display: grid
  line-height: 2rem
  grid-template-columns: 1fr 1fr 1fr
  grid-template-rows: 1fr
  grid-gap: 5px
  .copied-text
    #copied-message
      display: none
      text-align: center
      color: green
      padding: 5px 30px
      margin: 0 25%
      border-radius 5px
      background: #ded
      &.copied-message-visible
        display: block
  .search-filter
    text-align: right
    label:
      line-height: 3rem

.hidden-icon
  display: none